//
//  PCRegex.h
//  ObjCRegex
//
//  Created by Whitney Young on 8/2/06.
//  Copyright 2006 __MyCompanyName__. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import <ObjCRegex/pcre.h>

enum {
	PCCaseInsensitive = PCRE_CASELESS,
	PCMultiline = PCRE_MULTILINE,
	PCDotAll = PCRE_DOTALL,
	PCExtended = PCRE_EXTENDED,
	PCAnchored = PCRE_ANCHORED,
	PCDollarEndOnly = PCRE_DOLLAR_ENDONLY,
	PCExtra = PCRE_EXTRA,
	PCNoTBOL = PCRE_NOTBOL,
	PCNoEOL = PCRE_NOTEOL,
	PCUnGreedy = PCRE_UNGREEDY,
	PCNotEmpty = PCRE_NOTEMPTY,
	PCUTF8 = PCRE_UTF8,
	PCNoAutoCapture = PCRE_NO_AUTO_CAPTURE,
	PCNoUTF8Check = PCRE_NO_UTF8_CHECK,
	PCAutoCallout = PCRE_AUTO_CALLOUT,
	PCPartial = PCRE_PARTIAL,
	PCDFAShortest = PCRE_DFA_SHORTEST,
	PCDFARestart = PCRE_DFA_RESTART,
	PCFirstline = PCRE_FIRSTLINE,
	PCDupnames = PCRE_DUPNAMES,
	PCNewlineCR = PCRE_NEWLINE_CR,
	PCNewlineLF = PCRE_NEWLINE_LF,
	PCNewlineCRLF = PCRE_NEWLINE_CRLF
};

@interface PCRegex : NSObject {
	pcre *regex;
	pcre_extra *extra;
	NSString *pattern;
	int backrefCount;
}

+ (id)regexWithPattern:(NSString *)pattern;
+ (id)regexWithPattern:(NSString *)pattern options:(int)options;

- (id)initWithPattern:(NSString *)pattern;
- (id)initWithPattern:(NSString *)pattern options:(int)options;

- (NSString *)pattern;

@end

@interface PCRegexBackreference : NSObject {
	int offset;
	int length;
	NSString *string;
}

- (unsigned)offset;
- (unsigned)length;
- (NSString *)string;

@end

@interface NSString (PCRegexMatching)
- (BOOL)match:(PCRegex *)regex;
- (BOOL)match:(PCRegex *)regex backreferences:(NSArray **)arrayRef;
- (BOOL)matchPattern:(NSString *)pattern;
- (BOOL)matchPattern:(NSString *)pattern backreferences:(NSArray **)arrayRef;

- (BOOL)match:(PCRegex *)regex backreferences:(NSArray **)arrayRef inRange:(NSRange)range;
- (BOOL)matchPattern:(NSString *)pattern backreferences:(NSArray **)arrayRef inRange:(NSRange)range;
@end

//examples
//
//#import <ObjCRegex/ObjCRegex.h>
//
//PCRegex *regex = [PCRegex regexWithPattern:@"(te)st" options:0];
//NSArray *array;
//NSLog(@"match? %i", [@"testing" match:regex backreferences:&array]);
//NSLog(@"what was matched: %@, %@", [[array objectAtIndex:0] string], [[array objectAtIndex:1] string]);
//
//NSLog(@"match? %i", [@"this a test of my regex engine" matchPattern:@"[^ ]+.a .... (of)" backreferences:&array]);
//NSLog(@"what was matched: %@, %@", [[array objectAtIndex:0] string], [[array objectAtIndex:1] string]);
//
//
//int start = 0;
//NSMutableString *string = [NSMutableString stringWithString:@"blue clue glue"];
//regex = [PCRegex regexWithPattern:@"[bcg]lue" options:0];
//
//while ([string match:regex backreferences:&array inRange:NSMakeRange(start, [string length] - start)]) {
//	int offset = [[array objectAtIndex:0] offset];
//	int length = [[array objectAtIndex:0] length];
//	
//	if ([[[array objectAtIndex:0] string] isEqualToString:@"blue"]) {
//		NSString *newString = @"replacement";
//		[string replaceCharactersInRange:NSMakeRange(offset, length) withString:newString];
//		length = [newString length];
//	} else if ([[[array objectAtIndex:0] string] isEqualToString:@"clue"]) {
//		NSString *newString = @"via regex";
//		[string replaceCharactersInRange:NSMakeRange(offset, length) withString:newString];
//		length = [newString length];			
//	} else if ([[[array objectAtIndex:0] string] isEqualToString:@"glue"]) {
//		NSString *newString = @"is fun";
//		[string replaceCharactersInRange:NSMakeRange(offset, length) withString:newString];
//		length = [newString length];			
//	}
//	
//	start = offset + length;
//	NSLog(@"matched %@", [[array objectAtIndex:0] string]);
//}
//NSLog(string);